unit Unit2;

interface

uses
  Windows,Messages{, CommCtrl};

type
  NMHDR = record
    hWndFrom : HWND;
    idFrom   : UINT;
    code     : UINT;
  end;
  PNMHDR = ^NMHDR;

  TtcItem = packed record
    mask        : UINT;
    lpReserved1 : UINT;
    lpReserved2 : UINT;
    pszText     : PAnsiChar;
    cchTextMax  : integer;
    iImage      : integer;
    lParam      : LPARAM;
  end;

  TTabChange = procedure (hTabCtrl:HWND; PageID:integer);

var
  TabGetFocus      : TTabChange;
  TabLostFocus     : TTabChange;
  IsSetTabCtrlProc : boolean = False;

const
  ICC_TAB_CLASSES = $00000008; // tab, tooltips
  
  // tab control style
  TCS_SCROLLOPPOSITE    = $0001;
  TCS_BOTTOM            = $0002;
  TCS_RIGHT             = $0002;
  TCS_FORCEICONLEFT     = $0010;
  TCS_FORCELABELLEFT    = $0020;
  TCS_HOTTRACK          = $0040;
  TCS_VERTICAL          = $0080;
  TCS_TABS              = $0000;
  TCS_BUTTONS           = $0100;
  TCS_SINGLELINE        = $0000;
  TCS_MULTILINE         = $0200;
  TCS_RIGHTJUSTIFY      = $0000;
  TCS_FIXEDWIDTH        = $0400;
  TCS_RAGGEDRIGHT       = $0800;
  TCS_FOCUSONBUTTONDOWN = $1000;
  TCS_OWNERDRAWFIXED    = $2000;
  TCS_TOOLTIPS          = $4000;
  TCS_FOCUSNEVER        = $8000;

  //
  TCIF_TEXT       = $0001;
  TCIF_IMAGE      = $0002;
  TCIF_RTLREADING = $0004;
  TCIF_PARAM      = $0008;

  // tab control messages
  TCM_FIRST          = $1300;
  TCM_GETITEMA       = TCM_FIRST + 5;
  TCM_SETITEMA       = TCM_FIRST + 6;
  TCM_INSERTITEMA    = TCM_FIRST + 7;
  TCM_GETITEMW       = TCM_FIRST + 60;
  TCM_SETITEMW       = TCM_FIRST + 61;
  TCM_INSERTITEMW    = TCM_FIRST + 62;
  TCM_GETITEM        = TCM_GETITEMA;
  TCM_SETITEM        = TCM_SETITEMA;
  TCM_INSERTITEM     = TCM_INSERTITEMA;

  TCM_GETIMAGELIST   = TCM_FIRST + 2;
  TCM_SETIMAGELIST   = TCM_FIRST + 3;
  TCM_GETITEMCOUNT   = TCM_FIRST + 4;
  TCM_DELETEITEM     = TCM_FIRST + 8;
  TCM_DELETEALLITEMS = TCM_FIRST + 9;
  TCM_GETITEMRECT    = TCM_FIRST + 10;
  TCM_GETCURSEL      = TCM_FIRST + 11;
  TCM_SETCURSEL      = TCM_FIRST + 12;
  TCM_HITTEST        = TCM_FIRST + 13;
  TCM_SETITEMEXTRA   = TCM_FIRST + 14;
  TCM_ADJUSTRECT     = TCM_FIRST + 40;
  TCM_SETITEMSIZE    = TCM_FIRST + 41;
  TCM_REMOVEIMAGE    = TCM_FIRST + 42;
  TCM_SETPADDING     = TCM_FIRST + 43;
  TCM_GETROWCOUNT    = TCM_FIRST + 44;
  TCM_GETTOOLTIPS    = TCM_FIRST + 45;
  TCM_SETTOOLTIPS    = TCM_FIRST + 46;
  TCM_GETCURFOCUS    = TCM_FIRST + 47;
  TCM_SETCURFOCUS    = TCM_FIRST + 48;

  // tab control notify
  TCN_FIRST       = 0-550;
  TCN_KEYDOWN     = TCN_FIRST - 0;
  TCN_SELCHANGE   = TCN_FIRST - 1;
  TCN_SELCHANGING = TCN_FIRST - 2;

function tcCreate(hParent:HWND; Left, Top, Width, Height:integer; tcExStyle, tcStyle:Cardinal):HWND; stdcall;
procedure tcTabAdd(hTabCtrl:HWND; Title:PAnsiChar; Index:integer); stdcall;
function tcGetCurrentPage(hTabCtrl:HWND):integer; stdcall;
procedure SetChangeTabProc(tgf:TTabChange; tlf:TTabChange); stdcall;

procedure InitCommonControls; stdcall; external 'comctl32.dll' name 'InitCommonControls';

implementation

procedure SetChangeTabProc(tgf:TTabChange; tlf:TTabChange); stdcall;
begin
  TabGetFocus:=tgf;
  TabLostFocus:=tlf;
end;

function TabCtrlProc(hWnd:hWnd;Msg,wParam,lParam:Longint):Longint; stdcall;
begin
  Result:=0;
  case Msg of
    WM_NOTIFY: begin
      case integer(PNMHDR(lParam)^.Code) of
        TCN_SELCHANGE   : if @TabGetFocus<>nil then TabGetFocus(PNMHDR(lParam)^.hWndFrom,tcGetCurrentPage(PNMHDR(lParam)^.hWndFrom));
        TCN_SELCHANGING : if @TabLostFocus<>nil then TabLostFocus(PNMHDR(lParam)^.hWndFrom,tcGetCurrentPage(PNMHDR(lParam)^.hWndFrom));
        else Result:=CallWindowProc(Pointer(GetWindowLong(hWnd,GWL_USERDATA)),hWnd,Msg,wParam,lParam);
      end;
    end;
    //WM_KEYDOWN: begin //   ,    
    //  if wParam = VK_TAB then
    //end;
    else Result:=CallWindowProc(Pointer(GetWindowLong(hWnd,GWL_USERDATA)),hWnd,Msg,wParam,lParam);
  end;
end;

function tcCreate(hParent:HWND; Left, Top, Width, Height:integer; tcExStyle, tcStyle:Cardinal):HWND; stdcall;
begin
  Result:=0;
  try
    Result:=CreateWindowEx(tcExStyle,'SysTabControl32',nil,WS_CHILD {or WS_VISIBLE} or WS_TABSTOP or tcStyle,Left,Top,Width,Height,hParent,0,hInstance,nil);
    SendMessage(Result,WM_SETFONT,GetStockObject(DEFAULT_GUI_FONT),0);
    if not IsSetTabCtrlProc then begin
      SetWindowLong(hParent,GWL_USERDATA,SetWindowLong(hParent,GWL_WNDPROC,LongInt(@TabCtrlProc)));
      IsSetTabCtrlProc:=True;
    end;
    ShowWindow(Result,SW_SHOWNORMAL);
    //InvalidateRect(Result,nil,False);
    //UpdateWindow(Result);
  except
  end;
end;

procedure tcTabAdd(hTabCtrl:HWND; Title:PAnsiChar; Index:integer); stdcall;
var
  tcItem:TtcItem;
begin
  tcItem.mask:=TCIF_TEXT or TCIF_IMAGE;
  tcItem.iImage:=-1;
  tcItem.pszText:=Title;
  SendMessage(hTabCtrl,TCM_INSERTITEM,Index,Longint(@tcItem));
end;

function tcGetCurrentPage(hTabCtrl:HWND):integer; stdcall;
begin
  Result:=SendMessage(hTabCtrl,TCM_GETCURSEL,0,0);
end;

{procedure InitTabCtrl;
var
  cc:TInitCommonControlsEx;
begin
  cc.dwSize:=SizeOf(cc);
  cc.dwICC:=ICC_TAB_CLASSES;
  InitCommonControlsEx(cc);
end;}

initialization
//  InitTabCtrl;
//  InitCommonControls;
  TabGetFocus:=nil;
  TabLostFocus:=nil;

end.
